import PropTypes from 'prop-types';
import { ReactNode, ReactElement } from 'react';

declare enum StatusEnum {
    ERROR = "error",
    IDLE = "idle",
    LOADING = "loading",
    SUCCESS = "success"
}

declare class PromiseType<T = unknown> extends Promise<T> {
    cancel?: () => void;
}
declare type OnErrorFn = ((err: Error) => void | Promise<void>) | undefined;
declare type OnSuccessFn<T = unknown> = ((result: T) => void | Promise<void>) | undefined;
declare type OnCancelFn = (() => void | Promise<void>) | undefined;
/**
 * @typedef {Object} UseDataLoaderConfig
 * @property {Function} [onSuccess] callback when a request success
 * @property {Function} [onError] callback when a error is occured, this will override the onError specified on the Provider if any
 * @property {*} [initialData] initial data if no one is present in the cache before the request
 * @property {number} [pollingInterval] relaunch the request after the last success
 * @property {boolean} [enabled=true] launch request automatically (default true)
 * @property {boolean} [keepPreviousData=true] do we need to keep the previous data after reload (default true)
 */
interface UseDataLoaderConfig<T = unknown> {
    enabled?: boolean;
    initialData?: T;
    keepPreviousData?: boolean;
    onError?: OnErrorFn;
    onSuccess?: OnSuccessFn;
    pollingInterval?: number;
}
/**
 * @typedef {Object} UseDataLoaderResult
 * @property {boolean} isIdle true if the hook in initial state
 * @property {boolean} isLoading true if the request is launched
 * @property {boolean} isSuccess true if the request success
 * @property {boolean} isError true if the request throw an error
 * @property {boolean} isPolling true if the request if enabled is true, pollingInterval is defined and the status is isLoading or isSuccess
 * @property {*} previousData if keepPreviousData is true it return the last data fetched
 * @property {*} data initialData if no data is fetched or not present in the cache otherwise return the data fetched
 * @property {string} error the error occured during the request
 * @property {Function} reload reload the data
 */
interface UseDataLoaderResult<T = unknown> {
    data?: T;
    error?: Error;
    isError: boolean;
    isIdle: boolean;
    isLoading: boolean;
    isPolling: boolean;
    isSuccess: boolean;
    previousData?: T;
    reload: () => Promise<void>;
}

declare type DataLoaderConstructorArgs<T = unknown> = {
    enabled?: boolean;
    key: string;
    method: () => PromiseType<T>;
    pollingInterval?: number;
    keepPreviousData?: boolean;
    notify: (updatedRequest: DataLoader<T>) => void;
};
declare class DataLoader<T = unknown> {
    key: string;
    status: StatusEnum;
    pollingInterval?: number;
    private notify;
    method: () => PromiseType<T>;
    private cancelMethod?;
    keepPreviousData?: boolean;
    private errorListeners;
    private successListeners;
    private cancelListeners;
    error?: Error;
    timeout?: number;
    constructor(args: DataLoaderConstructorArgs<T>);
    launch: () => Promise<void>;
    cancel: () => Promise<void>;
    addOnSuccessListener(fn: OnSuccessFn<T>): void;
    addOnErrorListener(fn: OnErrorFn): void;
    addOnCancelListener(fn: OnCancelFn): void;
    removeOnSuccessListener(fn: OnSuccessFn<T>): void;
    removeOnErrorListener(fn: OnErrorFn): void;
    removeOnCancelListener(fn: OnCancelFn): void;
    destroy(): void;
}

declare type CachedData = Record<string, unknown>;
declare type Reloads = Record<string, () => Promise<void | unknown>>;
declare type UseDataLoaderInitializerArgs<T = unknown> = {
    enabled?: boolean;
    key: string;
    status?: StatusEnum;
    method: () => PromiseType<T>;
    pollingInterval?: number;
    keepPreviousData?: boolean;
};
interface Context {
    addCachedData: (key: string, newData: unknown) => void;
    addReload: (key: string, method: () => Promise<void | unknown>) => void;
    addRequest: (key: string, args: UseDataLoaderInitializerArgs) => DataLoader;
    cacheKeyPrefix: string;
    onError?: (error: Error) => void | Promise<void>;
    clearAllCachedData: () => void;
    clearAllReloads: () => void;
    clearCachedData: (key?: string) => void;
    clearReload: (key?: string) => void;
    getCachedData: (key?: string) => unknown | CachedData;
    getReloads: (key?: string) => (() => Promise<void | unknown>) | Reloads;
    getRequest: (key: string) => DataLoader | undefined;
    reload: (key?: string) => Promise<void>;
    reloadAll: () => Promise<void>;
}
declare const DataLoaderProvider: {
    ({ children, cacheKeyPrefix, onError, }: {
        children: ReactNode;
        cacheKeyPrefix: string;
        onError: OnErrorFn;
    }): ReactElement;
    propTypes: {
        cacheKeyPrefix: PropTypes.Requireable<string>;
        children: PropTypes.Validator<string | number | boolean | {} | PropTypes.ReactElementLike | PropTypes.ReactNodeArray>;
        onError: PropTypes.Requireable<(...args: any[]) => any>;
    };
    defaultProps: {
        cacheKeyPrefix: undefined;
        onError: undefined;
    };
};
declare const useDataLoaderContext: () => Context;

/**
 * @param {string} key key to save the data fetched in a local cache
 * @param {() => PromiseType} method a method that return a promise
 * @param {useDataLoaderConfig} config hook configuration
 * @returns {useDataLoaderResult} hook result containing data, request state, and method to reload the data
 */
declare const useDataLoader: <T>(fetchKey: string, method: () => PromiseType<T>, { enabled, initialData, keepPreviousData, onError, onSuccess, pollingInterval, }?: UseDataLoaderConfig<T>) => UseDataLoaderResult<T>;

export { DataLoaderProvider, useDataLoader, useDataLoaderContext };
